// Number display control for pong board

#include <avr/pgmspace.h>

prog_uint8_t digit[10][3] =
{  { 0b11111, 0b10001, 0b11111 },	// 0
   { 0b10001, 0b11111, 0b10000 },	// 1
   { 0b11101, 0b10101, 0b10111 },	// 2
   { 0b10101, 0b10101, 0b11111 },	// 3
   { 0b00111, 0b00100, 0b11111 },	// 4
   { 0b10111, 0b10101, 0b11101 },	// 5
   { 0b11111, 0b10101, 0b11101 },	// 6
   { 0b00001, 0b00001, 0b11111 },	// 7
   { 0b11111, 0b10101, 0b11111 },	// 8
   { 0b10111, 0b10101, 0b11111 }, 	// 9
};

#define CharHight 5					
#define DisplayWidth 12		
#define DisplayHight 10	

void InitMatrix()
{	TCCR0 |= _BV(CS01); 	// F_CPU/8 as interrupt clock
	TIMSK |= _BV(TOIE0); 	// timer for overflow interrupt	
	TIFR  |= _BV(TOV0); 	// clear overflow flag
}

unsigned long LED[DisplayWidth];  	// Video RAM
char TextBuffer[4] = "\0"; 			

void ClearMatrix()						 
{for (int i=0; i<DisplayWidth; LED[i++]=0);}

void WriteMatrix(char *chr, int column, int line)
{ int i, f,	ReCo; char *text;
  ReCo = 0b00011111 << (CharHight-line);  
  for (text=chr; (*text); text++)			   
    {	f = *text-'0';	   	
		for (i=0 ; i<3 ; i++)					
		{   if ((column >= 0) && (column < DisplayWidth))							
			{ 	LED[column] &= ~ReCo;									
		  		LED[column] |= pgm_read_byte(&digit[f][i]) << (CharHight-line); 
			}
			column++;														
		}
		column++;  															
  	}
}

// interrupt routine for display refresh
ISR (SIG_OVERFLOW0)
{	unsigned int lv, C, D; static unsigned int cl = 0;
	cli();
	PORTB=PORTC=PORTD=0;
	if (cl==DisplayWidth) cl=0;
	if (cl==0) PORTB &=~16; else PORTB |=16;         
	PORTB |=8; PORTB &=~8; PORTB |=4; PORTB &=~4; lv=LED[cl++];
	D=lv&255; C=D&15; D&=240;
	PORTD=D&255; PORTC=C&255; PORTB=(lv>>8)&3;  	
	sei();
}
